#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_Icon=iso.ico
#AutoIt3Wrapper_Outfile=H:\Scripts_and_Such\AutiIT3_scripts\Tur-MakePS3ISO\Tur-MakePS3ISO.exe
#AutoIt3Wrapper_UseUpx=y
#AutoIt3Wrapper_Res_Fileversion=1.0.0.0
#AutoIt3Wrapper_Res_LegalCopyright= Turranius 2015
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****

#include <File.au3>
#include <Array.au3>
#include <Date.au3>

#include <ButtonConstants.au3>
#include <GUIConstantsEx.au3>
#include <ProgressConstants.au3>
#include <WindowsConstants.au3>

#cs
	1.0 = * Initial release.
#ce

Global $Version = "1.0"

TraySetToolTip("Tur-MakePS3ISO " & $Version)

HotKeySet("+{ESC}", "_Quit")

Global $IniFile = @ScriptDir & "\Tur-MakePS3ISO.ini"
Global $LogFile = @ScriptDir & "\Tur-MakePS3ISO.log"

MsgBox(0, "Tur-MakePS3ISO " & $Version & " by Turranius", "We will now ask you a number of questions." & @CRLF & @CRLF & "Please see the title of each box that pops up for what we are asking for." & @CRLF & @CRLF & "You can press Shift-Escape at any time to quit/abort (if there is a messagebox up, you have to click on it before it exits).")

Global $SourceFolder = ""
Global $DestinationFolder = ""

If FileExists($IniFile) Then
	$SourceFolder = IniRead($IniFile, "General", "SourceFolder", "")
	$DestinationFolder = IniRead($IniFile, "General", "DestinationFolder", "")
EndIf

$SourceFolder = FileSelectFolder("Select source folder. Single game or folder containing games to do them all.", $SourceFolder)
If $SourceFolder = "" Then
	MsgBox(16, "Error", "Got no location. Aborting")
	Exit
EndIf

$DestinationFolder = FileSelectFolder("Select destination folder", $DestinationFolder)
If $DestinationFolder = "" Then
	MsgBox(16, "Error", "Got no location. Aborting")
	Exit
EndIf

Global $Delete_PS3_UPDATE = MsgBox(36, "Delete PS3_UPDATE?", "Do you want to delete the PS3_UPDATE from the source game before creating the ISO?" & @CRLF & @CRLF & "This is generally safe and will save about 200MB per game.")
If $Delete_PS3_UPDATE = "6" Then
	$Delete_PS3_UPDATE = "TRUE"
EndIf

Global $Delete_Source_on_Success = MsgBox(36, "Delete Source?", "Do you want to delete the source game if the conversion was a success?")
If $Delete_Source_on_Success = "6" Then
	$Delete_Source_on_Success = "TRUE"
EndIf

Global $Split_ISO = MsgBox(36, "Split ISO?", "Do you want to split the ISO into 4GB chunks? Only good if you are going to store them on a FAT32 disk.")
If $Split_ISO = "6" Then
	Global $SPLIT = "TRUE"
	Global $EXE = "genps3iso_v2.5.exe -s"
Else
	Global $SPLIT = "FALSE"
	Global $EXE = "genps3iso_v2.5.exe"
EndIf

IniWrite($IniFile, "General", "SourceFolder", $SourceFolder)
IniWrite($IniFile, "General", "DestinationFolder", $DestinationFolder)

$FileList = _FileListToArray($SourceFolder & "\", "*")
If @error = 1 Then
	MsgBox(0, "", "No Files Found in " & $FileList & "\")
	Exit
EndIf
_ArraySort($FileList, 0, 1)

For $i = 1 To $FileList[0]
	$FileName = $FileList[$i]
	;msgbox(0, $i, $FileName)
	If $FileName = "PS3_DISC.SFB" Then
		MsgBox(0, "PS3_DISC.SFB Found!", "Single game conversion: " & $SourceFolder)
		ConvertToISO($SourceFolder, $DestinationFolder)
		Exit
	EndIf
Next

_ArrayDisplay($FileList, $FileList[0] & " games to be converted. Press X to start or Shift-ESC to quit.", "1:", 16, -1, "FolderName", Default, 0xDDFFDD)
For $i = 1 To $FileList[0]
	$FileName = $FileList[$i]

	If FileExists($SourceFolder & "\" & $FileName & "\PS3_DISC.SFB") And FileExists($SourceFolder & "\" & $FileName & "\PS3_GAME") Then
		ConvertToISO($SourceFolder & "\" & $FileName, $DestinationFolder)
	Else
		ConsoleWrite("Missing files - Skipping " & $FileName & " as it is missing PS3_GAME and/or PS3_DISC.SFB" & @CRLF)
		FileWriteLine($LogFile, _NowDate() & " " & _NowTime(3) & " Missing files - Skipping " & $FileName & " as it is missing PS3_GAME and/or PS3_DISC.SFB")
		FileWriteLine($LogFile, "")
		MsgBox(16, "Missing files", "Skipping " & $FileName & " as it is missing PS3_GAME and/or PS3_DISC.SFB", 10)
	EndIf
Next

Func ConvertToISO($SourceFolder, $DestinationFolder)

	$sFileNameArray = StringSplit($SourceFolder, "\")
	For $i = 1 To $sFileNameArray[0]
		$sFileName = $sFileNameArray[$i]
	Next

	If FileExists($DestinationFolder & "\" & $sFileName & ".iso") Then
		ConsoleWrite("Already exists - " & $sFileName & ".iso already exists. Skipping." & @CRLF)
		MsgBox(16, "Already exists", $sFileName & ".iso already exists. Skipping.", 10)
	Else
		If $Delete_PS3_UPDATE = "TRUE" Then
			If FileExists($SourceFolder & "\PS3_UPDATE") Then
				DirRemove($SourceFolder & "\PS3_UPDATE", 1)
			EndIf
		EndIf

		TrayTip($sFileName & ".iso", "Starting", 10)
		ConsoleWrite($sFileName & ".iso - Starting" & @CRLF)
		TraySetToolTip($sFileName & ".iso")

		Local $ReturnCode = RunWait($EXE & " " & Chr(34) & $SourceFolder & Chr(34) & " " & Chr(34) & $DestinationFolder & "\" & $sFileName & ".iso" & Chr(34), "", @SW_MINIMIZE)
		If $ReturnCode <> "0" Then
			If $SPLIT = "FALSE" Then
				If FileExists($DestinationFolder & "\" & $sFileName & ".iso") Then
					ConsoleWrite("Deleting " & $DestinationFolder & "\" & $sFileName & ".iso" & @CRLF)
					FileDelete($DestinationFolder & "\" & $sFileName & ".iso")
				EndIf
			Else
				If FileExists($DestinationFolder & "\" & $sFileName & ".iso.0") Then
					ConsoleWrite("Deleting " & $DestinationFolder & "\" & $sFileName & ".iso.*" & @CRLF)
					FileDelete($DestinationFolder & "\" & $sFileName & ".iso.*")
				EndIf
			EndIf

			If $ReturnCode = "-1073741510" Then
				FileWriteLine($LogFile, _NowDate() & " " & _NowTime(3) & " Aborted: " & $sFileName)
				ConsoleWrite($sFileName & @CRLF & @CRLF & "Aborted - Removing iso file if it exists. Press Shift-ESC NOW to abort the whole program, then press OK, or just OK to continue." & @CRLF)
				MsgBox(16, "Aborted", $sFileName & @CRLF & @CRLF & "Aborted - Removing iso file if it exists. Press Shift-ESC NOW to abort the whole program, then press OK, or just OK to continue.", 10)
			ElseIf $ReturnCode = "-32767" Then
				ConsoleWrite($sFileName & @CRLF & @CRLF &  "Input Error - Removing iso file if it exists. Press Shift-ESC NOW to abort the whole program, then press OK, or just OK to continue." & @CRLF)
				MsgBox(16, "Input Error", $sFileName & @CRLF & @CRLF &  "Input Error - Removing iso file if it exists. Press Shift-ESC NOW to abort the whole program, then press OK, or just OK to continue.", 10)
				FileWriteLine($LogFile, _NowDate() & " " & _NowTime(3) & " Input Error: " & $sFileName)
			ElseIf $ReturnCode = "-32768" Then
				ConsoleWrite($sFileName & @CRLF & @CRLF &  "Output Error - Removing iso file if it exists. Press Shift-ESC NOW to abort the whole program, then press OK, or just OK to continue." & @CRLF)
				MsgBox(16, "Output Error", $sFileName & @CRLF & @CRLF &  "Output Error - Removing iso file if it exists. Press Shift-ESC NOW to abort the whole program, then press OK, or just OK to continue.", 10)
				FileWriteLine($LogFile, _NowDate() & " " & _NowTime(3) & " Output Error: " & $sFileName)
			Else
				ConsoleWrite("Unknown ReturnCode (" & $ReturnCode & ") for " & $sFileName & @CRLF)
				FileWriteLine($LogFile, _NowDate() & " " & _NowTime(3) & " Unknown ReturnCode (" & $ReturnCode & ") for " & $sFileName)
				MsgBox(16, "Unknown ReturnCode", $ReturnCode & @CRLF & $sFileName, 60 )
			EndIf
			FileWriteLine($LogFile, _NowDate() & " " & _NowTime(3) & " " & Chr(34) & @ScriptDir & "\" & $EXE & Chr(34) & " " & Chr(34) & $SourceFolder & Chr(34) & " " & Chr(34) & $DestinationFolder & "\" & $sFileName & ".iso" & Chr(34))
			FileWriteLine($LogFile, _NowDate() & " " & _NowTime(3) & " You may test manually by pasting the above line into a CMD window.")
			FileWriteLine($LogFile, "")
		Else ; Returncode is 0 so should be OK. Make sure we actually have an ISO just in case...
			If $SPLIT = "FALSE" Then
				If FileExists($DestinationFolder & "\" & $sFileName & ".iso") Then
					ConsoleWrite($sFileName & ".iso - Success" & @CRLF)
					TrayTip($sFileName & ".iso", "Success", 10)
					If $Delete_Source_on_Success = "TRUE" Then
						DirRemove($SourceFolder, 1)
					EndIf
				Else
					ConsoleWrite("ReturnCode was 0 but " & $sFileName & ".iso does not exist.. ?!" & @CRLF)
					FileWriteLine($LogFile, _NowDate() & " " & _NowTime(3) & " ReturnCode was 0 but " & $sFileName & ".iso does not exist.. ?!")
					MsgBox(16, "Error.", "ReturnCode was 0 but " & $sFileName & ".iso does not exist.. ?!")
				EndIf
			Else
				If FileExists($DestinationFolder & "\" & $sFileName & ".iso.0") Then
					TrayTip($sFileName & ".iso.0+", "Success", 10)
					If $Delete_Source_on_Success = "TRUE" Then
						DirRemove($SourceFolder, 1)
					EndIf
				Else
					ConsoleWrite("ReturnCode was 0 but " & $sFileName & ".iso.0 does not exist.. ?!" & @CRLF)
					FileWriteLine($LogFile, _NowDate() & " " & _NowTime(3) & " ReturnCode was 0 but " & $sFileName & ".iso.0 does not exist.. ?!")
					MsgBox(16, "Error.", "ReturnCode was 0 but " & $sFileName & ".iso.0 does not exist.. ?!")
				EndIf
			EndIf
		EndIf
	EndIf
EndFunc   ;==>ConvertToISO


; This function is called whenever you quit the script.
Func _Quit()
	Exit
EndFunc   ;==>_Quit
